//@author: dottore
//@description: Draws a surface using the data position texture. shading by phong directional
//@tags: 3d surface
//@credits: 
// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD ;        //the models world matrix
float4x4 tV: VIEW ;         //view matrix as set via Renderer (EX9)
float4x4 tVP: VIEWPROJECTION ;
float4x4 tWV: WORLDVIEW ;
float4x4 tWVP: WORLDVIEWPROJECTION ;
float4x4 matVI: VIEWINVERSE ;

#define NUM_ITERATIONS 16
#define NUM_ITERATIONS_RELIEF1 11
#define NUM_ITERATIONS_RELIEF2 5

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{ 
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
	addressU = wrap;
};
texture TexBump <string uiname="Bump Map";>;
sampler2D cone_map = sampler_state
{
	Texture = <TexBump>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};
texture TexNorm <string uiname="Normal Map";>;
sampler2D normal_map = sampler_state
{
	Texture = <TexNorm>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};
sampler3D sphere_map = sampler_state
{
	Texture = <TexBump>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

texture TexTransform <string uiname="Transform Texture";>;
sampler SampTransform = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexTransform);          //apply a texture to the sampler
    MipFilter = none;                    //sampler states
    MinFilter = none;
    MagFilter = none;
};
texture pTexTransform <string uiname="previous Transform Texture";>;
sampler pSampTransform = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (pTexTransform);          //apply a texture to the sampler
    MipFilter = none;                    //sampler states
    MinFilter = none;
    MagFilter = none;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;
float4x4 ptWVP <string uiname="Previus Frame WorldViewProj Transform";>;
float velGain <string uiname="Velocity Gain";> = 1;
float3 mbcorr = float3(0.0039,-0.0075,-0.1);
bool isPTxCd <string uiname="Position as Texture Coordinates";> = 0;

#include "VSM.fxh"
#include "Blinn_Specular_VSM.fxh"
#define minTwoPi -6.283185307179586476925286766559
#define TwoPi 6.283185307179586476925286766559

float depth = 0.1;
float normamount = 1;
bool DEPTH_BIAS = false;
bool BORDER_CLAMP = false;
float3 size_source;

float3 cone(float2 uv0, float3 eyeVec, sampler2D cone_map)
{
	float3 rayPos;
	float3 rayVec;
	rayPos = float3(uv0, 0.0);
	
	rayVec = normalize(eyeVec);
	rayVec.z = abs(rayVec.z);

	// DerayPosth bias used by Fabio rayPosolicarrayPoso
	if (DEPTH_BIAS)
	{
		float db = 1.0 - rayVec.z;
		db *= db;
		db = 1.0 - db*db;
		rayVec.xy *= db;
	}
	
	rayVec.xy *= depth;

	float dist = length(rayVec.xy);
	
	for( int i=0;i<NUM_ITERATIONS; i++ )
	{
		float4 tex = tex2D(cone_map, rayPos.xy);
		float height = saturate(tex.w - rayPos.z);

		float cone_ratio = tex.z*tex.z;
		
		//float stepDist = height * cone_ratio / (rayVec.z*cone_ratio + dist);
		float stepDist = height * cone_ratio / (cone_ratio + dist);
		rayPos += rayVec * stepDist;
		
		///1
		// float4 tex = tex2D(cone_map, rayPos.xy);
		// float height = max(0, tex.w - rayPos.z);
		// float stepDist = height * 0.2;
		// rayPos += rayVec * stepDist;
		
		//float stepDist = height * cone_ratio / (cone_ratio + 0.1);
		///2
		//rayPos += rayVec * height * cone_ratio / (rayVec.z*cone_ratio + dist);
		
	}
	
	return rayPos.xyz;
}

float4x4 Vrotate(float rotX, 
		 float rotY, 
		 float rotZ)
  {
   rotX *= TwoPi;
   rotY *= TwoPi;
   rotZ *= TwoPi;
   float sx = sin(rotX);
   float cx = cos(rotX);
   float sy = sin(rotY);
   float cy = cos(rotY);
   float sz = sin(rotZ);
   float cz = cos(rotZ);

   return float4x4( cz*cy+sz*sx*sy, sz*cx, cz*-sy+sz*sx*cy, 0,
                   -sz*cy+cz*sx*sy, cz*cx, sz*sy+cz*sx*cy , 0,
                    cx * sy       ,-sx   , cx * cy        , 0,
                    0             , 0    , 0              , 1);
  }
// -----------------------------------------------------------------------------
// STRUCT:
// -----------------------------------------------------------------------------
struct appdata
{
    float4 PosO: POSITION;
    float3 NormO: NORMAL;
    float4 PrID: TEXCOORD0;
    float2 TexCdO: TEXCOORD1;
    float3 tang: TANGENT;
    float3 bino: BINORMAL;
};

struct vs2ps2
{
    float4 PosWVP: POSITION ;
    float4 PosW: TEXCOORD0 ;
    float3 PosWV : TEXCOORD1 ;
    float3 NormWV: NORMAL ;
	float2 TexCdShadow : TEXCOORD2 ;
    float3 LightDirWV: TEXCOORD3 ;
    float3 ViewDirWV: TEXCOORD4 ;
	float2 TexCd: TEXCOORD5 ;
	float3 eyeVec: TEXCOORD6 ;
	float3 lightVec: TEXCOORD7 ;
    float4 vel : COLOR0 ;
};

// -----------------------------------------------------------------------------
// VERTEXSHADERS:
// -----------------------------------------------------------------------------

// PLACE and DEFORM technique
vs2ps2 VS(appdata In)
{
    //inititalize all fields of output struct with 0
    vs2ps2 Out = (vs2ps2)0;

    float3 TRowId = float3(0.1667, 0.5, 0.8333);
	float4 TRow1 = tex2Dlod(SampTransform, float4(In.PrID.x, TRowId.x, 0, 0));
	float4 TRow2 = tex2Dlod(SampTransform, float4(In.PrID.x, TRowId.y, 0, 0));
	float4 TRow3 = tex2Dlod(SampTransform, float4(In.PrID.x, TRowId.z, 0, 0));
	/*float4 pTRow1 = tex2Dlod(pSampTransform, float4(In.PrID.x, TRowId.x, 0, 0));
	float4 pTRow2 = tex2Dlod(pSampTransform, float4(In.PrID.x, TRowId.y, 0, 0));
	float4 pTRow3 = tex2Dlod(pSampTransform, float4(In.PrID.x, TRowId.z, 0, 0));*/
	float4 prePos = In.PosO;
	//float4 pprePos = In.PosO;
	float3 preNorm = In.NormO;
	//ApplyCenter
	prePos.xyz -= float3(TRow3.g, TRow3.b, TRow3.a);
	//pprePos.xyz -= float3(pTRow3.g, pTRow3.b, pTRow3.a);
	//Apply scale
	prePos.xyz *= float3(TRow1.a, TRow2.r, TRow2.g);
	//pprePos.xyz *= float3(pTRow1.a, pTRow2.r, pTRow2.g);
	//Apply rotation
	prePos = mul(prePos, Vrotate(TRow2.b, TRow2.a, TRow3.r));
	//pprePos = mul(pprePos, Vrotate(pTRow2.b, pTRow2.a, pTRow3.r));
    preNorm = mul(preNorm, Vrotate(TRow2.b, TRow2.a, TRow3.r));
	//Apply translate
	prePos.xyz += TRow1.rgb;
	//pprePos.xyz += pTRow1.rgb;

    Out.PosW = mul(prePos, tW);
   	 Out.PosWV = mul(prePos, tWV);
   	Out.PosWVP = mul(prePos, tWVP);
	if(isPTxCd) Out.TexCd = mul(In.PosO, tTex).xy;
	else Out.TexCd = mul(In.TexCdO, tTex);
	float3 npos = Out.PosWVP.xyz;
	//float3 pnpos = mul(pprePos, ptWVP).xyz;
	float3 pnpos = mul(prePos, ptWVP).xyz;
	Out.vel.rgb = (npos - pnpos) * velGain + mbcorr;
	Out.vel.a = 1;
    //normal in view space
    Out.NormWV = normalize(mul(preNorm, tWV));
   
    float3x3 tangentMap = float3x3(In.tang, In.bino, preNorm);
    tangentMap = mul(tangentMap, tW);
	float3 eyeVec = Out.PosW - matVI[3].xyz;	
	Out.eyeVec = mul(tangentMap, eyeVec);
	
    //inverse light direction in view space
    float3 LightDirW = normalize(lPos - Out.PosW);
    Out.LightDirWV = mul(LightDirW, tV);
	Out.lightVec = mul(tangentMap, LightDirW);

	//Out.ViewDirV = -normalize(mul(Out.PosW, tWV));
    Out.ViewDirWV = -normalize(mul(In.PosO, tWV));
   
	// VSM texture coordinates:
    float4 surf_tex = mul(float4(Out.PosW.xyz, 1.0), light_tVP);
	surf_tex = surf_tex / surf_tex.w; 
	
	// Rescale viewport to be [0,1] (texture coordinate space)
	Out.TexCdShadow = surf_tex.xy * float2(0.5, -0.5) + 0.5;
	
    return Out;
}

// -----------------------------------------------------------------------------
// MRT STRUCT:
// -----------------------------------------------------------------------------
struct col
{
    float4 color : COLOR0 ;
    float4 space : COLOR1 ;
    float4 normal : COLOR2 ;
    float4 vel : COLOR3 ;
};


// -----------------------------------------------------------------------------
// PIXELSHADERS:
// -----------------------------------------------------------------------------

col PS1(vs2ps2 In): COLOR
{
    col c = (col)0;
	float3 normb = In.NormWV;
	float2 itexcd = In.TexCd;
	itexcd.x *= -1;
	if(depth!=0) normb += ((2 * (tex2D(normal_map,itexcd))) - 1.0)*-depth;
	float3 uvb = cone(itexcd, In.eyeVec, cone_map);
	c.color = Point_BlinnVSMcubemapSpecular(
		lPos,
		lCol,
		SpecAmount,
		In.PosW,
		normb,
		In.ViewDirWV,
		In.LightDirWV,
		uvb.xy
	);
	c.color.a = tex2D(Samp,itexcd).a;
	
	//POSITION
    c.space.xyz = In.PosWV;
    c.space.w   = 1.0f;
   
    //NORMALS
    float3 norm = normb;
    c.normal = float4(norm, gi);
	c.vel = In.vel;
	c.vel.a = 1;

    return c;
}

col PS2(vs2ps2 In): COLOR
{
    col c = (col)0;
	float3 normb = In.NormWV;
	float2 itexcd = In.TexCd;
	itexcd.x *= -1;
	if(depth!=0) normb += ((2 * (tex2D(normal_map,itexcd))) - 1.0)*-depth*normamount;
	float3 uvb = cone(itexcd, In.eyeVec, cone_map);
	c.color = Spot_BlinnVSMspecular(
    	lPos,
    	lCol,
    	SpecAmount,
		In.PosW,
    	normb,
    	In.ViewDirWV,
    	In.LightDirWV,
    	In.TexCdShadow,
    	uvb.xy
    );
	c.color.a = tex2D(Samp,itexcd).a;
	
	//POSITION
   	c.space.xyz = In.PosWV;
	if(depth!=0) c.space.xyz += normb * uvb.z * (-1*pow(depth,.5));
    c.space.w   = 1.0f;
    
    //NORMALS
    float3 norm = normb;
    c.normal = float4(norm, gi);
	c.vel = In.vel;

    return c;
}

col PS3(vs2ps2 In): COLOR
{
    col c = (col)0;
	float3 normb = In.NormWV;
	float2 itexcd = In.TexCd;
	itexcd.x *= -1;
	if(depth!=0) normb += ((2 * (tex2D(normal_map,itexcd))) - 1.0)*-depth;
	float3 uvb = cone(itexcd, In.eyeVec, cone_map);
	c.color = Directional_BlinnVSMspecular(
    	In.PosW,
    	normb,
    	In.ViewDirWV,
    	In.LightDirWV,
    	In.TexCdShadow,
    	uvb.xy
    );
	c.color.a = tex2D(Samp,itexcd).a;
	
	//POSITION
    c.space.xyz = In.PosWV;
	if(depth!=0) c.space.xyz += normb * uvb.z * (-1*pow(depth,.5));
    c.space.w   = 1.0f;
    
    //NORMALS
    float3 norm = normb;
    c.normal = float4(norm, gi);
	c.vel = In.vel;

    return c;
}

// -----------------------------------------------------------------------------
// TECHNIQUES:
// -----------------------------------------------------------------------------

technique Phong_PointLight
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
    	//ALPHABLENDENABLE = FALSE;
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS1();
    }
}

technique Phong_SpotLight
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
    	ALPHABLENDENABLE = FALSE;
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS2();
    }
}

technique Phong_DirectionalLight
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
    	ALPHABLENDENABLE = FALSE;
        VertexShader = compile vs_3_0 VS();
        PixelShader  = compile ps_3_0 PS3();
    }
}



