
//------------------------------------------------------------------------------
// Untweakables
//------------------------------------------------------------------------------
float4x4 tW : WORLD ;
float4x4 tV : VIEW ;
float4x4 tVP : VIEWPROJECTION ;
#define minTwoPi -6.283185307179586476925286766559
#define TwoPi 6.283185307179586476925286766559

float3 lightPos;
float2 lightRange;
texture TexYResampSpl <string uiname="Y Resampled Texture (spline)";>;
sampler YResampSpl = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexYResampSpl);          //apply a texture to the sampler
    MipFilter = linear;         //sampler states
    MinFilter = linear;
    MagFilter = linear;
    AddressU = clamp;//mirror;
    AddressV = clamp;//mirror;
};
texture TexDirSpl <string uiname="Direction Texture";>;
sampler DirSpl = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexDirSpl);          //apply a texture to the sampler
    MipFilter = linear;         //sampler states
    MinFilter = linear;
    MagFilter = linear;
    AddressU = clamp;//mirror;
    AddressV = clamp;//mirror;
};
float4x4 TransformW <string uiname="Mesh Transform";>;

float4x4 Vrotate(float rotX, 
		 float rotY, 
		 float rotZ)
  {
   rotX *= TwoPi;
   rotY *= TwoPi;
   rotZ *= TwoPi;
   float sx = sin(rotX);
   float cx = cos(rotX);
   float sy = sin(rotY);
   float cy = cos(rotY);
   float sz = sin(rotZ);
   float cz = cos(rotZ);

   return float4x4( cz*cy+sz*sx*sy, sz*cx, cz*-sy+sz*sx*cy, 0,
                   -sz*cy+cz*sx*sy, cz*cx, sz*sy+cz*sx*cy , 0,
                    cx * sy       ,-sx   , cx * cy        , 0,
                    0             , 0    , 0              , 0);
  }

//------------------------------------------------------------------------------
// VERTEX SHADER
//------------------------------------------------------------------------------
struct appdata
{
    float4 PosO: POSITION;
    float2 TexCdO: TEXCOORD0;
};
struct vs2ps
{
	float4 PosWVP    : POSITION ;
	float depth      : TEXCOORD0 ;
};

vs2ps VS(appdata In)
{
	vs2ps Out   = (vs2ps)0;
	
	float4 prePos = mul(In.PosO, TransformW);
	
	// Placement UV
    float4 PlaceCoords = float4(In.TexCdO.x, prePos.z+0.5, 0, 1);
	// get position and direction data from textures
    float4 particleData = tex2Dlod(YResampSpl, PlaceCoords);
    float3 SplineDirection = tex2Dlod(DirSpl, PlaceCoords);
	// get pitch and yaw from direction vector
    float pitch = asin(SplineDirection.y)/TwoPi;
    float yaw = atan2(-SplineDirection.x, -SplineDirection.z)/ TwoPi;
	// Evaluate rotation matrix
    float4x4 rotMatrix = Vrotate(pitch, yaw,0);
	// POSITION
	float4 cprePos = 0;
	cprePos.yx = prePos * particleData.a;
    cprePos.z = 0;
    float4 splPos = mul(cprePos, rotMatrix) + float4(particleData.xyz,1);
	
	float4 PosW   = mul(splPos, tW);
    Out.PosWVP    = mul(PosW, tVP);
    Out.depth = length(lightPos - PosW);//(length(lightPos - PosW)-lightRange.x)*lightRange.y;
    
	return Out;
}

//------------------------------------------------------------------------------
// PIXEL SHADER
//------------------------------------------------------------------------------

float4 PS_A(vs2ps In): COLOR
{	
	// Work out the depth of this fragment from the light, normalized to 0->1
	float distNormalized = (In.depth-lightRange.x)*lightRange.y;
	float2 depth;
    depth.x = distNormalized;
    depth.y = distNormalized.x * distNormalized.x;

    return depth.xyxy;
}

//------------------------------------------------------------------------------
// Techniques
//------------------------------------------------------------------------------

technique VSM_A_Depth
{
    pass P0
    {
        ALPHABLENDENABLE = FALSE;
        VertexShader = compile vs_3_0 VS();
        PixelShader = compile ps_3_0 PS_A();
    }
}

