
//------------------------------------------------------------------------------
// Untweakables
//------------------------------------------------------------------------------
float4x4 tW : WORLD ;
float4x4 tV : VIEW ;
float4x4 tVP : VIEWPROJECTION ;

float3 lightPos;
float2 lightRange;

texture TexTransform <string uiname="Transform Texture";>;
sampler SampTransform = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (TexTransform);          //apply a texture to the sampler
    MipFilter = none;                    //sampler states
    MinFilter = none;
    MagFilter = none;
};
#define minTwoPi -6.283185307179586476925286766559
#define TwoPi 6.283185307179586476925286766559

float4x4 Vrotate(float rotX, 
         float rotY, 
         float rotZ)
  {
   rotX *= TwoPi;
   rotY *= TwoPi;
   rotZ *= TwoPi;
   float sx = sin(rotX);
   float cx = cos(rotX);
   float sy = sin(rotY);
   float cy = cos(rotY);
   float sz = sin(rotZ);
   float cz = cos(rotZ);

   return float4x4( cz*cy+sz*sx*sy, sz*cx, cz*-sy+sz*sx*cy, 0,
                   -sz*cy+cz*sx*sy, cz*cx, sz*sy+cz*sx*cy , 0,
                    cx * sy       ,-sx   , cx * cy        , 0,
                    0             , 0    , 0              , 1);
  }

//------------------------------------------------------------------------------
// VERTEX SHADER
//------------------------------------------------------------------------------
struct vs2ps
{
	float4 PosWVP    : POSITION ;
	float depth      : TEXCOORD0 ;
};
struct appdata
{
    float4 PosO: POSITION;
    float4 PrID: TEXCOORD0;
};

vs2ps VS(appdata In)
{
	vs2ps Out   = (vs2ps)0;

    float3 TRowId = float3(0.1667, 0.5, 0.8333);
    float4 TRow1 = tex2Dlod(SampTransform, float4(In.PrID.x, TRowId.x, 0, 0));
    float4 TRow2 = tex2Dlod(SampTransform, float4(In.PrID.x, TRowId.y, 0, 0));
    float4 TRow3 = tex2Dlod(SampTransform, float4(In.PrID.x, TRowId.z, 0, 0));
	
    float4 prePos = In.PosO;
    //ApplyCenter
    prePos.xyz -= float3(TRow3.g, TRow3.b, TRow3.a);
    //Apply scale
    prePos.xyz *= float3(TRow1.a, TRow2.r, TRow2.g);
    //Apply rotation
    prePos = mul(prePos, Vrotate(TRow2.b, TRow2.a, TRow3.r));
    //Apply translate
    prePos.xyz += TRow1.rgb;
	
	float4 PosW = mul(prePos, tW);
    Out.PosWVP = mul(PosW, tVP);
    Out.depth = length(lightPos - PosW);//(length(lightPos - PosW)-lightRange.x)*lightRange.y;
    
	return Out;
}

//------------------------------------------------------------------------------
// PIXEL SHADER
//------------------------------------------------------------------------------

float4 PS_A(vs2ps In): COLOR
{	
	// Work out the depth of this fragment from the light, normalized to 0->1
	float distNormalized = (In.depth-lightRange.x)*lightRange.y;
	float2 depth;
    depth.x = distNormalized;
    depth.y = distNormalized.x * distNormalized.x;

    return depth.xyxy;
}

//------------------------------------------------------------------------------
// Techniques
//------------------------------------------------------------------------------

technique VSM_A_Depth
{
    pass P0
    {
        ALPHABLENDENABLE = FALSE;
        VertexShader = compile vs_3_0 VS();
        PixelShader = compile ps_3_0 PS_A();
    }
}

