//@author: micro.D
//@help: RGB = CurrentFrame.XYZ - PrevFrame.XYZ
//@tags: direction, velocity
//@credits:

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 ptWpr <string uiname="Previous frame preW";>;
float4x4 ctWpr <string uiname="Current frame preW";>;
float4x4 ptWps <string uiname="Previous frame postW";>;
float4x4 ctWps <string uiname="Current frame postW";>;
float4x4 ptVP <string uiname="Previous frame VP";>;
float4x4 ctVP <string uiname="Current frame VP";>;
float Gain = 1;
float radius = 1;
float PRadius = 1;
float amount = 1;
float PAmount = 1;
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

struct vs2ps
{
    float4 Pos : POSITION;
    float2 TexCd : TEXCOORD0;
    float4 col : COLOR0;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
float3 sphericalInverse3(float3 InCd, float rad) {
	float rd = pow(InCd.x,2)+pow(InCd.y,2)+pow(InCd.z,2);
	float3 OutCd = InCd;
	OutCd.x = (pow(rad,2)*InCd.x)/rd;
	OutCd.y = (pow(rad,2)*InCd.y)/rd;
	OutCd.z = (pow(rad,2)*InCd.z)/rd;
	return OutCd;
}
float4 sphericalInverse4(float4 InCd, float rad) {
	float rd = pow(InCd.x,2)+pow(InCd.y,2)+pow(InCd.z,2);
	float4 OutCd = InCd;
	OutCd.x = (pow(rad,2)*InCd.x)/rd;
	OutCd.y = (pow(rad,2)*InCd.y)/rd;
	OutCd.z = (pow(rad,2)*InCd.z)/rd;
	return OutCd;
}
vs2ps VSc(
    float4 Pos : POSITION,
    float2 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
	float4 Pos1 = lerp(mul(Pos, ctWpr), sphericalInverse4(mul(Pos,ctWpr),radius), amount);
    float4 p1=mul(Pos1, mul(ctWps, ctVP));
	float4 Pos2 = lerp(mul(Pos, ptWpr), sphericalInverse4(mul(Pos,ptWpr),PRadius), PAmount);
    float4 p2=mul(Pos2, mul(ptWps, ptVP));
    Out.Pos = p1;
    Out.col.r = .5 + (p1.x - p2.x)*Gain;
    Out.col.g = .5 + (p1.y - p2.y)*Gain;
    Out.col.b = .5 + (p1.z - p2.z)*Gain;
    Out.col.a = 1;

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
};
vs2ps VSp(
    float4 Pos : POSITION,
    float2 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
	float4 Pos1 = lerp(mul(Pos, ctWpr), sphericalInverse4(mul(Pos,ctWpr),radius), amount);
    float4 p1=mul(Pos1, mul(ctWps, ctVP));
	float4 Pos2 = lerp(mul(Pos, ptWpr), sphericalInverse4(mul(Pos,ptWpr),PRadius), PAmount);
    float4 p2=mul(Pos2, mul(ptWps, ptVP));
    Out.Pos = p2;
    Out.col.r = .5 + (p1.x - p2.x)*Gain;
    Out.col.g = .5 + (p1.y - p2.y)*Gain;
    Out.col.b = .5 + (p1.z - p2.z)*Gain;
    Out.col.a = 1;

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
};
vs2ps VSe(
    float4 Pos : POSITION,
    float2 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
	float4 Pos1 = lerp(mul(Pos, ctWpr), sphericalInverse4(mul(Pos,ctWpr),radius), amount);
    float4 p1=mul(Pos1, mul(ctWps, ctVP));
	float4 Pos2 = lerp(mul(Pos, ptWpr), sphericalInverse4(mul(Pos,ptWpr),PRadius), PAmount);
    float4 p2=mul(Pos2, mul(ptWps, ptVP));
    Out.Pos = p1+(p1-p2)/2;
    Out.col.r = .5 + (p1.x - p2.x)*Gain;
    Out.col.g = .5 + (p1.y - p2.y)*Gain;
    Out.col.b = .5 + (p1.z - p2.z)*Gain;
    Out.col.a = 1;

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}
vs2ps VSa(
    float4 Pos : POSITION,
    float2 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
	float4 Pos1 = lerp(mul(Pos, ctWpr), sphericalInverse4(mul(Pos,ctWpr),radius), amount);
    float4 p1=mul(Pos1, mul(ctWps, ctVP));
	float4 Pos2 = lerp(mul(Pos, ptWpr), sphericalInverse4(mul(Pos,ptWpr),PRadius), PAmount);
    float4 p2=mul(Pos2, mul(ptWps, ptVP));
    Out.Pos = (p1+p2)/2;
    Out.col.r = .5 + (p1.x - p2.x)*Gain;
    Out.col.g = .5 + (p1.y - p2.y)*Gain;
    Out.col.b = .5 + (p1.z - p2.z)*Gain;
    Out.col.a = 1;

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);

    return Out;
}

// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    //In.TexCd = In.TexCd / In.TexCd.w; // for perpective texture projections (e.g. shadow maps) ps_2_0

    float4 col = In.col;
    col.a = 1;
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique NextFrameEstimation
{
    pass P0
    {
        VertexShader = compile vs_3_0 VSc();
        PixelShader = compile ps_3_0 PS();
    }
	pass P1
    {
        VertexShader = compile vs_3_0 VSp();
        PixelShader = compile ps_3_0 PS();
    }
	pass P2
    {
        VertexShader = compile vs_3_0 VSe();
        PixelShader = compile ps_3_0 PS();
    }
}
technique AverageOfFrames
{
    pass P0
    {
        VertexShader = compile vs_3_0 VSc();
        PixelShader = compile ps_3_0 PS();
    }
	pass P1
    {
        VertexShader = compile vs_3_0 VSp();
        PixelShader = compile ps_3_0 PS();
    }
	pass P2
    {
        VertexShader = compile vs_3_0 VSa();
        PixelShader = compile ps_3_0 PS();
    }
}