#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "VectorFieldLines", Category = "3d", Version = "1.0", Help = "Basic template with one value in/out", Tags = "")]
	#endregion PluginInfo
	public class C1_03dVectorFieldLinesNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Coordinate X", DefaultValue = 0.0)]
		ISpread<double> FFixX;
		
		
		[Input("Coordinate Y", DefaultValue = 0.0)]
		ISpread<double> FFixY;
		
		[Input("Coordinate Z", DefaultValue = 0.0)]
		ISpread<double> FFixZ;
		
		[Input("Phases", DefaultValue = 0.0)]
		ISpread<double> FPhases;
		
		[Input("Inner Frequency", DefaultValue = 5.0, IsSingle = true)]
		ISpread<double> FInnerFrequency;
		
		[Input("Outer Frequency", DefaultValue = 8.0, IsSingle = true)]
		ISpread<double> FOuterFrequency;
		
		[Input("Field Strength", DefaultValue = 0.0, IsSingle = true)]
		ISpread<double> FFieldStrength;
		
		[Input("Time Damp", DefaultValue = 0.0, IsSingle = true)]
		ISpread<double> FTimeDamp;
		
		[Input("Translation Damp", DefaultValue = 0.0, IsSingle = true)]
		ISpread<double> FTranslationDamp;
		
		[Input("Attraction", DefaultValue = 0.0, IsSingle = true)]
		ISpread<double> FAttraction;
		
		[Input("Line Length", DefaultValue = 0, IsSingle = true)]
		ISpread<int> FPoints;
		
		[Output("Output X")]
		ISpread<ISpread<double>> FCoordinateX;
		
		[Output("Output Y")]
		ISpread<ISpread<double>> FCoordinateY;
		
		[Output("Output Z")]
		ISpread<ISpread<double>> FCoordinateZ;
		
		#endregion fields & pins
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			int i,k;
			
			FCoordinateX.SliceCount = SpreadMax;
			FCoordinateY.SliceCount = SpreadMax;
			FCoordinateZ.SliceCount = SpreadMax;
			
			for(i = 0; i < SpreadMax; i++){
				FCoordinateX[i].SliceCount = FPoints[0];
				FCoordinateY[i].SliceCount = FPoints[0];
				FCoordinateZ[i].SliceCount = FPoints[0];
				FCoordinateX[i][0] = FFixX[i];
				FCoordinateY[i][0] = FFixY[i];
				FCoordinateZ[i][0] = FFixZ[i];
			}
			
			// for every line
			for(i = 0; i < FFixX.SliceCount; i++){
				// for every point in current line
				for(k = 1; k < FPoints[0]; k++){
					
					FCoordinateX[i][k] = vectorField(FCoordinateX[i][k],FCoordinateX[i][k-1],FCoordinateY[i][k-1],FCoordinateZ[i][k-1],FInnerFrequency[0],FOuterFrequency[0],FFieldStrength[0],FPhases,0,FTimeDamp[0],FAttraction[0]);
					FCoordinateY[i][k] = vectorField(FCoordinateY[i][k],FCoordinateY[i][k-1],FCoordinateZ[i][k-1],FCoordinateX[i][k-1],FInnerFrequency[0],FOuterFrequency[0],FFieldStrength[0],FPhases,1,FTimeDamp[0],FAttraction[0]);
					FCoordinateZ[i][k] = vectorField(FCoordinateZ[i][k],FCoordinateZ[i][k-1],FCoordinateX[i][k-1],FCoordinateY[i][k-1],FInnerFrequency[0],FOuterFrequency[0],FFieldStrength[0],FPhases,2,FTimeDamp[0],FAttraction[0]);
					
					/*
					FCoordinateX[i][k] = FCoordinateX[i][k]*tdamp + FCoordinateX[i][k-1]*attraction 
										 + Math.Sin( 
														Math.Sin ( FCoordinateZ[*innerFrequency + phases[1] )*
														Math.Sin ( FCoordinateY*innerFrequency + phases[2] )*
														outerFrequency + phases[3]
													)*fieldStrength;							
					*/
				}
			}	
		}
		
		double vectorField(double x, double xOld,double y,double z,double innerFrequency,double outerFrequency,double fieldStrength,ISpread<double> phases,int offset,double tdamp,double attraction)
		{
			return x*tdamp + xOld*attraction + Math.Sin( 
				Math.Sin ( z*innerFrequency + phases[0+offset*3] )*
				Math.Sin ( y*innerFrequency + phases[1+offset*3] )*
				outerFrequency + phases[2+offset*3])*fieldStrength*0.1
			;
		}
	}
}
