//@author: vvvv group
//@help: draws a mesh with a constant color
//@tags: template, basic
//@credits:

//
//--------------------------------------------------------------------------------------------------
// PARAMETERS:
// 

//--------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;   //projection matrix as set via Renderer (EX9)
float4x4 tWVP: WORLDVIEWPROJECTION;

//material properties
float fade = 0;
float hue = 0;
bool invert = 0;
float2 Move = 0;
float Zoom = 4;
//texture
texture tex1 <string uiname="Texture1";>;
sampler samp1 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (tex1);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};
texture tex2 <string uiname="Texture2";>;
sampler samp2 = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (tex2);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

float4x4 tTex1: TEXTUREMATRIX <string uiname="Texture Transform1";>;
float4x4 tTex2: TEXTUREMATRIX <string uiname="Texture Transform2";>;
float4x4 tCol;
//the data structure: vertexshader to pixelshader
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos : POSITION;
    float4 TexCd1 : TEXCOORD0;
    float4 TexCd2 : TEXCOORD1;
};

// 

//--------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// 

//--------------------------------------------------------------------------------------------------

vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd1 = mul(TexCd, tTex1);
    Out.TexCd2 = mul(TexCd, tTex2);

    return Out;
}

// 

//--------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// 

//--------------------------------------------------------------------------------------------------
float3 rgb2hsl(float3 c){float cmax=max(c.r,max(c.g,c.b)),cmin=min(c.r,min(c.g,c.b)),l=(cmax+cmin)/2,d=(cmax-cmin),s=l>.5?d/(2-cmax-cmin)/l/2:d/(cmax+cmin),h;float3 ch=(cmax==c);if(ch.r==ch.g&&ch.r==1)ch.rg=float2(1,0);if(ch.g==ch.b&&ch.g==1)ch.gb=float2(1,0);if(ch.b==ch.r&&ch.b==1)ch.br=float2(1,0);h=frac((dot(min(2,ch),(c.gbr-c.brg)/d+float3(0,2,4)))/6);if(cmax==cmin)h=s=0;return float3(h,s,l);}
float3 hsl2rgb(float3 h){return lerp(h.z,saturate((abs(frac(-h.x+float3(3,1,2)/3)*6-3)-1))*h.z*2,saturate(h.y)*saturate(2-2*h.z));}
float4 colTr(float4 incol) {
	float3 hcol = rgb2hsl(incol.rgb);
	hcol.r *= hue;
	float4 pcol = {0,0,0,1};
	pcol.rgb = hsl2rgb(hcol);
	float4 col = mul(pcol, tCol);
	if(invert) col=1-col;
	col.a=1;
	return col;
}
float4 blend(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
    col = tex2D(samp1, In.TexCd1) * (1-fade) + tex2D(samp2, In.TexCd2) * fade;
    return colTr(col);
}
float4 mul(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,tx1*tx2,min(fade*2,1));
	}
	else {
		col=lerp(tx1*tx2,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 screen(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,1-(1-tx1)*(1-tx2),min(fade*2,1));
	}
	else {
		col=lerp(1-(1-tx1)*(1-tx2),tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 darken(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,min(tx1,tx2),min(fade*2,1));
	}
	else {
		col=lerp(min(tx1,tx2),tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 lighten(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,max(tx1,tx2),min(fade*2,1));
	}
	else {
		col=lerp(max(tx1,tx2),tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 difference(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,abs(tx1-tx2),min(fade*2,1));
	}
	else {
		col=lerp(abs(tx1-tx2),tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 negation(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,1-abs(1-tx1-tx2),min(fade*2,1));
	}
	else {
		col=lerp(1-abs(1-tx1-tx2),tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 exclusion(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,tx1+tx2-2*tx1*tx2,min(fade*2,1));
	}
	else {
		col=lerp(tx1+tx2-2*tx1*tx2,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 overlayL(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr = {0,0,0,1};
	txr.r= (tx1.r < 0.5) ? (2*tx1.r*tx2.r) : (1-2*(1-tx1.r)*(1-tx2.r));
	txr.g= (tx1.g < 0.5) ? (2*tx1.g*tx2.g) : (1-2*(1-tx1.g)*(1-tx2.g));
	txr.b= (tx1.b < 0.5) ? (2*tx1.b*tx2.b) : (1-2*(1-tx1.b)*(1-tx2.b));
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 overlayR(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr = {0,0,0,1};
	txr.r= (tx2.r < 0.5) ? (2*tx2.r*tx1.r) : (1-2*(1-tx2.r)*(1-tx1.r));
	txr.g= (tx2.g < 0.5) ? (2*tx2.g*tx1.g) : (1-2*(1-tx2.g)*(1-tx1.g));
	txr.b= (tx2.b < 0.5) ? (2*tx2.b*tx1.b) : (1-2*(1-tx2.b)*(1-tx1.b));
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 softlightL(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr = {0,0,0,1};
	txr.r= (tx2.r < 0.5) ? (2*tx1.r*tx2.r + pow(tx1.r,2) * (1-2*tx2.r)) : (sqrt(tx1.r)*

(2*tx2.r-1) + (2*tx1.r) * (1-tx2.r));
	txr.g= (tx2.g < 0.5) ? (2*tx1.g*tx2.g + pow(tx1.g,2) * (1-2*tx2.g)) : (sqrt(tx1.g)*

(2*tx2.g-1) + (2*tx1.g) * (1-tx2.g));
	txr.b= (tx2.b < 0.5) ? (2*tx1.b*tx2.b + pow(tx1.b,2) * (1-2*tx2.b)) : (sqrt(tx1.b)*

(2*tx2.b-1) + (2*tx1.b) * (1-tx2.b));
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 softlightR(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr = {0,0,0,1};
	txr.r= (tx1.r < 0.5) ? (2*tx2.r*tx1.r + pow(tx2.r,2) * (1-2*tx1.r)) : (sqrt(tx2.r)*

(2*tx1.r-1) + (2*tx2.r) * (1-tx1.r));
	txr.g= (tx1.g < 0.5) ? (2*tx2.g*tx1.g + pow(tx2.g,2) * (1-2*tx1.g)) : (sqrt(tx2.g)*

(2*tx1.g-1) + (2*tx2.g) * (1-tx1.g));
	txr.b= (tx1.b < 0.5) ? (2*tx2.b*tx1.b + pow(tx2.b,2) * (1-2*tx1.b)) : (sqrt(tx2.b)*

(2*tx1.b-1) + (2*tx2.b) * (1-tx1.b));
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 dodgeL(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr= {0,0,0,1};
	txr.r= (tx2.r == 1.0) ? 1.0 : tx1.r/(1-tx2.r);
	txr.g= (tx2.g == 1.0) ? 1.0 : tx1.g/(1-tx2.g);
	txr.b= (tx2.b == 1.0) ? 1.0 : tx1.b/(1-tx2.b);
	if (txr.r > 1.0){txr.r = 1.0;}
    if (txr.g > 1.0){txr.g = 1.0;}
    if (txr.b > 1.0){txr.b = 1.0;}
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 dodgeR(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr= {0,0,0,1};
	txr.r= (tx1.r == 1.0) ? 1.0 : tx2.r/(1-tx1.r);
	txr.g= (tx1.g == 1.0) ? 1.0 : tx2.g/(1-tx1.g);
	txr.b= (tx1.b == 1.0) ? 1.0 : tx2.b/(1-tx1.b);
	if (txr.r > 1.0){txr.r = 1.0;}
    if (txr.g > 1.0){txr.g = 1.0;}
    if (txr.b > 1.0){txr.b = 1.0;}
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 burnL(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
        float4 temp = {0,0,0,1};
        float4 temp1 = 1-(1-tx1)/tx2;
        temp.r = (temp1.r > 1) ? 1 : temp1.r;
        temp.g = (temp1.g > 1) ? 1 : temp1.g;
        temp.b = (temp1.b > 1) ? 1 : temp1.b;
	if (fade<0.5) {
		col=lerp(tx1,temp,min(fade*2,1));
	}
	else {
		col=lerp(temp,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 burnR(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
        float4 temp = {0,0,0,1};
        float4 temp1 = 1-(1-tx2)/tx1;
        temp.r = (temp1.r > 1) ? 1 : temp1.r;
        temp.g = (temp1.g > 1) ? 1 : temp1.g;
        temp.b = (temp1.b > 1) ? 1 : temp1.b;
	if (fade<0.5) {
		col=lerp(tx1,temp,min(fade*2,1));
	}
	else {
		col=lerp(temp,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 reflect(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr = {0,0,0,1};
	txr.r = (tx2.r == 1) ? 1 : pow(tx1.r,2)/(1-tx2.r);
	txr.g = (tx2.g == 1) ? 1 : pow(tx1.g,2)/(1-tx2.g);
	txr.b = (tx2.b == 1) ? 1 : pow(tx1.b,2)/(1-tx2.b);
	if (txr.r > 1.0){txr.r = 1.0;}
        if (txr.g > 1.0){txr.g = 1.0;}
        if (txr.b > 1.0){txr.b = 1.0;}
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 glow(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 txr = {0,0,0,1};
	txr.r = (tx2.r == 1) ? 1 : pow(tx2.r,2)/(1-tx1.r);
	txr.g = (tx2.g == 1) ? 1 : pow(tx2.g,2)/(1-tx1.g);
	txr.b = (tx2.b == 1) ? 1 : pow(tx2.b,2)/(1-tx1.b);
	if (txr.r > 1.0){txr.r = 1.0;}
        if (txr.g > 1.0){txr.g = 1.0;}
        if (txr.b > 1.0){txr.b = 1.0;}
	if (fade<0.5) {
		col=lerp(tx1,txr,min(fade*2,1));
	}
	else {
		col=lerp(txr,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 freeze(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,1-pow(1-tx1,2)/tx2,min(fade*2,1));
	}
	else {
		col=lerp(1-pow(1-tx1,2)/tx2,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 heat(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
        float4 temp = {0,0,0,1};
        float4 temp1 = 1-pow(1-tx2,2)/tx1;
        temp.r = (temp1.r > 1) ? 1 : temp1.r;
        temp.g = (temp1.g > 1) ? 1 : temp1.g;
        temp.b = (temp1.b > 1) ? 1 : temp1.b;
	if (fade<0.5) {
		col=lerp(tx1,temp,min(fade*2,1));
	}
	else {
		col=lerp(temp,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 add(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	if (fade<0.5) {
		col=lerp(tx1,tx1+tx2,min(fade*2,1));
	}
	else {
		col=lerp(tx1+tx2,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 xor(vs2ps In): COLOR
{
    float4 col = {0,0,0,1};
	float4 tx1 = tex2D(samp1, In.TexCd1);
	float4 tx2 = tex2D(samp2, In.TexCd2);
	float4 rst = ((1-floor(tx1*2)/2)*floor(tx2*2)/2)+((floor(tx1*2)/2)*(1-floor(tx2*2)/2))*1.5;
	if (fade<0.5) {
		col=lerp(tx1,rst,min(fade*2,1));
	}
	else {
		col=lerp(rst,tx2,max(fade*2-1,0));
	}
	col.a=1;
    return colTr(col);
}
float4 colwrap(vs2ps In): COLOR
{
    float4 tx1 = tex2D(samp1,In.TexCd1);
    float4 tx2 = tex2D(samp2,In.TexCd2);
    int4 ip;
    float4 col = lerp(max(modf(tx1 + fade, ip), tx1), tx2, fade);
    return colTr(col);
}
float4 morphR(vs2ps In): COLOR {
    float4 c0=tex2D(samp2,In.TexCd2);
    float4 c1=tex2D(samp1,In.TexCd1);
    float2 x=In.TexCd1;
    float2 x0=(x-.5)*pow(2,c1.z*Zoom)+.5+Move*2*normalize((c1.xy-.5))*pow(length((c1.xy-.5)),0.7);
    float2 x1=(x-.5)/pow(2,c0.z*Zoom)+.5+Move*2*normalize((c0.xy-.5))*pow(length((c0.xy-.5)),0.7);
    float4 a0=tex2D(samp2,lerp(x,x0,1-fade));
    float4 a1=tex2D(samp1,lerp(x1,x,1-fade));
    float4 c=lerp(a0,a1,1-fade);
    return colTr(c);
}
float4 morphL(vs2ps In): COLOR {
    float4 c0=tex2D(samp1,In.TexCd1);
    float4 c1=tex2D(samp2,In.TexCd2);
    float2 x=In.TexCd1;
    float2 x0=(x-.5)*pow(2,c1.z*Zoom)+.5+Move*2*normalize((c1.xy-.5))*pow(length((c1.xy-.5)),0.7);
    float2 x1=(x-.5)/pow(2,c0.z*Zoom)+.5+Move*2*normalize((c0.xy-.5))*pow(length((c0.xy-.5)),0.7);
    float4 a0=tex2D(samp1,lerp(x,x0,fade));
    float4 a1=tex2D(samp2,lerp(x1,x,fade));
    float4 c=lerp(a0,a1,fade);
    return colTr(c);
}
// 

//--------------------------------------------------------------------------------------------------
// TECHNIQUES:
// 

//--------------------------------------------------------------------------------------------------

technique _blend { 
	pass P0 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 blend();}
	pass P1 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 mul();}
	pass P2 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 screen();}
	pass P3 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 darken();}
	pass P4 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 lighten();}
	pass P5 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 difference();}
	pass P6 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 negation();}
	pass P7 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 exclusion();}
	pass P8 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 overlayL();}
	pass P9 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 overlayR();}
	pass P10 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 softlightL();}
	pass P11 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 softlightR();}
	pass P12 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 dodgeL();}
	pass P13 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 dodgeR();}
	pass P14 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 burnL();}
	pass P15 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 burnR();}
	pass P16 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 reflect();}
	pass P17 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 glow();}
	pass P18 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 freeze();}
	pass P19 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 heat();}
	pass P20 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 add();}
	pass P21 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 xor();}
	pass P22 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 colwrap();}
	pass P23 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 morphL();}
	pass P24 { VertexShader = compile vs_3_0 VS(); PixelShader = compile ps_3_0 morphR();}
}